package forumBeans;

import forumBeans.User;
import forumBeans.UserHome;

import javax.ejb.*;
import javax.sql.DataSource;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import java.sql.*;
import java.util.Collection;
import java.util.ArrayList;
import java.rmi.RemoteException;

public class ForumMessageBean implements EntityBean {

    private static final String DATA_SOURCE = "java:comp/env/jdbc/forum/ForumDS";

    private EntityContext mEntityContext;

    // Bean-managed state fields
    private Long mId;
    private User mUser;
    private String mSubject;
    private String mBody;

    public ForumMessageBean() {
        System.out.println("New forumBeans.ForumMessage bean created by EJB container.");
    }

    // Getter/setter methods on Entity Bean fields

    public Long getId() {
        return mId;
    }

    public User getUser() {
        return mUser;
    }

    public void setUser(User aUser) {
        mUser = aUser;
    }

    public String getSubject() {
        return mSubject;
    }

    public void setSubject(String aSubject) {
        mSubject = aSubject;
    }

    public String getBody() {
        return mBody;
    }

    public void setBody(String aBody) {
        mBody = aBody;
    }


    // EJB-required methods follow

    /**
     * Called by the container. Associates this bean instance with a particular context.
     */
    public void setEntityContext(EntityContext aEntityContext) throws EJBException {
        System.out.println("setEntityContext() called.");
        mEntityContext = aEntityContext;
    }

    /**
     * Called by Container.
     * Disassociates this bean instance with a particular context environment.
     */
    public void unsetEntityContext() throws EJBException {
        System.out.println("unsetEntityContext() called.");
        mEntityContext = null;
    }

    /**
     * This is the initialization method that corresponds
     * to the create() method in the Home Interface.
     *
     * When the client calls the Home Object's create() method,
     * the Home Object then calls this ejbCreate() method.
     *
     * @return The primary key for this user
     */
    public Long ejbCreate(User aUser, String aSubject, String aBody)
            throws CreateException {
        System.out.println("ejbCreate() called.");

        mUser = aUser;
        mSubject = aSubject;
        mBody = aBody;

        Long userId = null;
        try {
            userId = aUser.getId();
        } catch (RemoteException re) {
            throw new CreateException("Invalid user!");
        }

        Connection dbConn = null;
        PreparedStatement insertStatement = null;
        Statement selectIdentityStatement = null;
        try {
            dbConn = getConnection();
            insertStatement = dbConn.prepareStatement(
                "insert into T_MESSAGE (USER_ID, SUBJECT, BODY) values (?, ?, ?)");
            insertStatement.setString(1, userId.toString());
            insertStatement.setString(2, aSubject);
            insertStatement.setString(3, aBody);
            insertStatement.executeUpdate();

            selectIdentityStatement = dbConn.createStatement();
            ResultSet rs = selectIdentityStatement.executeQuery("SELECT @@IDENTITY");
            rs.next();
            mId = new Long(rs.getLong(1));
            return mId;

        } catch (SQLException sqle) {
            throw new EJBException(sqle.getMessage(), sqle);
        } finally {
            try {
                if (insertStatement != null)
                    insertStatement.close();
            } catch (Exception e) { }
            try {
                if (selectIdentityStatement != null)
                    selectIdentityStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    public void ejbPostCreate(User aUser, String aSubject, String aBody)
            throws CreateException {
        System.out.println("ejbPostCreate() called.");
    }

    /**
    * Removes entity bean data from the database.
    * Corresponds to when client calls home.remove().
    */
    public void ejbRemove() {
        System.out.println("ejbRemove() called.");

        Connection dbConn = null;
        PreparedStatement deleteStatement = null;
        try {
            Long id = (Long) mEntityContext.getPrimaryKey();
            dbConn = getConnection();
            deleteStatement = dbConn.prepareStatement(
                "delete from T_MESSAGE where ID = ?");
            deleteStatement.setString(1, id.toString());
            if (deleteStatement.executeUpdate() != 1) {
                throw new EJBException("Failed to delete message " + id + " from database");
            }
        } catch (SQLException e) {
            throw new EJBException(e.getMessage());
        } finally {
            try {
                if (deleteStatement != null)
                    deleteStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Called by the container. Updates the in-memory entity bean object
     * to reflect the current value stored in the database.
     */
    public void ejbLoad() {
        System.out.println("ejbLoad() called.");

        Connection dbConn = null;
        PreparedStatement selectStatement = null;
        try {
            Long id = (Long) mEntityContext.getPrimaryKey();
            dbConn = getConnection();
            selectStatement = dbConn.prepareStatement(
                "select * from T_MESSAGE where ID = ?");
            selectStatement.setString(1, id.toString());
            ResultSet rs = selectStatement.executeQuery();
            rs.next();
            mId = new Long(rs.getLong("ID"));
            long userId = rs.getLong("USER_ID");
            UserHome userHome = getUserHome();
            mUser = userHome.findByPrimaryKey(new Long(userId));
            mSubject = rs.getString("SUBJECT");
            mBody = rs.getString("BODY");
        } catch (Exception e) {
            throw new EJBException(e.getMessage(), e);
        } finally {
            try {
                if (selectStatement != null)
                    selectStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Called from the Container. Updates the database to reflect the current
     * values of this in-memory entity bean instance.
     */
    public void ejbStore() {
        System.out.println("ejbStore() called.");

        Connection dbConn = null;
        PreparedStatement updateStatement = null;
        try {
            Long id = (Long) mEntityContext.getPrimaryKey();
            dbConn = getConnection();
            updateStatement = dbConn.prepareStatement(
                "update T_MESSAGE set USER_ID = ?, SUBJECT = ?, BODY = ? where ID = ?");
            updateStatement.setString(1, mUser.getId().toString());
            updateStatement.setString(2, mSubject);
            updateStatement.setString(3, mBody);
            updateStatement.setString(4, id.toString());
            updateStatement.executeUpdate();
        } catch (Exception e) {
            throw new EJBException(e.getMessage(), e);
        } finally {
            try {
                if (updateStatement != null)
                    updateStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Called by Container. Implementation can acquire needed resources.
     */
    public void ejbActivate() throws EJBException {
        System.out.println("ejbActivate() called.");
        mId = (Long) mEntityContext.getPrimaryKey();
    }

    /**
     * Called by Container. Releases held resources for passivation.
     */
    public void ejbPassivate() throws EJBException {
        System.out.println("ejbPassivate () called.");
        mId = null;
    }

    /**
     * Finds a forumBeans.ForumMessage by its primary key.
     */
    public Long ejbFindByPrimaryKey(Long aKey)
            throws FinderException {
        System.out.println("ejbFindByPrimaryKey(" + aKey + ") called.");

        Connection dbConn = null;
        PreparedStatement selectStatement = null;
        try {
            dbConn = getConnection();
            selectStatement = dbConn.prepareStatement(
                "select ID from T_MESSAGE where ID = ?");
            selectStatement.setString(1, aKey.toString());
            ResultSet rs = selectStatement.executeQuery();
            if (!rs.next())
                throw new ObjectNotFoundException("Could not find message with ID " + aKey);
            return aKey;
        } catch (SQLException sqle) {
            throw new EJBException(sqle.getMessage(), sqle);
        } finally {
            try {
                if (selectStatement != null)
                    selectStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Finds all forumBeans.ForumMessage-s.
     */
    public Collection ejbFindAll()
            throws FinderException {
        System.out.println("ejbFindAll() called.");

        Connection dbConn = null;
        Statement selectStatement = null;
        try {
            dbConn = getConnection();
            selectStatement = dbConn.createStatement();
            ResultSet rs = selectStatement.executeQuery("select ID from T_MESSAGE");
            ArrayList resultIdList = new ArrayList();
            while (rs.next()) {
                Long id = new Long(rs.getString("ID"));
                resultIdList.add(id);
            }
            return resultIdList;
        } catch (SQLException sqle) {
            throw new FinderException(sqle.getMessage());
        } finally {
            try {
                if (selectStatement != null)
                    selectStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Gets connection from the Container's datasource.
     */
    private Connection getConnection()
            throws SQLException {
        DataSource ds = null;
        try {
            Context context = new InitialContext();
            ds = (DataSource) context.lookup(DATA_SOURCE);
            Connection dbConnection = ds.getConnection();
            return dbConnection;
        } catch (NamingException ne) {
            throw new SQLException("Can not find datasource " + DATA_SOURCE);
        }
    }

    public static UserHome getUserHome() throws NamingException {
        InitialContext ctx = new InitialContext();
        UserHome userHome = (UserHome)PortableRemoteObject.narrow(
                ctx.lookup("ejb/forum/UserEJB"), UserHome.class);
        return userHome;
    }

}
