package forumBeans;

import javax.ejb.*;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.sql.DataSource;
import java.sql.*;
import java.util.Collection;
import java.util.ArrayList;

public class UserBean implements EntityBean {

    private static final String DATA_SOURCE = "java:comp/env/jdbc/forum/ForumDS";

    private EntityContext mEntityContext;

    // Bean-managed state fields
    private Long mId;
    private String mLogin;
    private String mPassword;
    private String mFullName;

    public UserBean() {
        System.out.println("New User entity bean created by EJB container.");
    }


    // Getter/setter methods on Entity Bean fields

    public Long getId() {
        return mId;
    }

    public String getLogin() {
        return mLogin;
    }

    public void setLogin(String aLogin) {
        mLogin = aLogin;
    }

    public String getPassword() {
        return mPassword;
    }

    public void setPassword(String aPassword) {
        mPassword = aPassword;
    }

    public String getFullName() {
        return mFullName;
    }

    public void setFullName(String aFullName) {
        mFullName = aFullName;
    }


    // EJB-required methods follow

    /**
     * Called by the container. Associates this bean instance with a particular context.
     */
    public void setEntityContext(EntityContext aEntityContext) throws EJBException {
        System.out.println("setEntityContext() called.");
        mEntityContext = aEntityContext;
    }

    /**
     * Called by Container.
     * Disassociates this bean instance with a particular context environment.
     */
    public void unsetEntityContext() throws EJBException {
        System.out.println("unsetEntityContext() called.");
        mEntityContext = null;
    }

    /**
     * This is the initialization method that corresponds
     * to the create() method in the Home Interface.
     *
     * When the client calls the Home Object's create() method,
     * the Home Object then calls this ejbCreate() method.
     *
     * @return The primary key for this user
     */
    public Long ejbCreate(String aLogin, String aPassword, String aFullName)
            throws CreateException {
        System.out.println("ejbCreate() called.");

        mLogin = aLogin;
        mPassword = aPassword;
        mFullName = aFullName;

        Connection dbConn = null;
        PreparedStatement insertStatement = null;
        Statement selectIdentityStatement = null;
        try {
            dbConn = getConnection();
            insertStatement = dbConn.prepareStatement(
                "insert into T_USER (LOGIN, PASSWORD, FULL_NAME) values (?, ?, ?)");
            insertStatement.setString(1, mLogin);
            insertStatement.setString(2, mPassword);
            insertStatement.setString(3, mFullName);
            insertStatement.executeUpdate();

            selectIdentityStatement = dbConn.createStatement();
            ResultSet rs = selectIdentityStatement.executeQuery("SELECT @@IDENTITY");
            rs.next();
            mId = new Long(rs.getLong(1));
            return mId;

        } catch (SQLException sqle) {
            throw new EJBException(sqle.getMessage(), sqle);
        } finally {
            try {
                if (insertStatement != null)
                    insertStatement.close();
            } catch (Exception e) { }
            try {
                if (selectIdentityStatement != null)
                    selectIdentityStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    public void ejbPostCreate(String aLogin, String aPassword, String aFullName)
            throws CreateException {
        System.out.println("ejbPostCreate() called.");
    }

    /**
    * Removes entity bean data from the database.
    * Corresponds to when client calls home.remove().
    */
    public void ejbRemove() {
        System.out.println("ejbRemove() called.");

        Connection dbConn = null;
        PreparedStatement deleteStatement = null;
        try {
            Long id = (Long) mEntityContext.getPrimaryKey();
            dbConn = getConnection();
            deleteStatement = dbConn.prepareStatement(
                "delete from T_USER where ID = ?");
            deleteStatement.setString(1, id.toString());
            if (deleteStatement.executeUpdate() != 1) {
                throw new EJBException("Failed to delete user " + id + " from database");
            }
        } catch (SQLException e) {
            throw new EJBException(e.getMessage());
        } finally {
            try {
                if (deleteStatement != null)
                    deleteStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Called by the container. Updates the in-memory entity bean object
     * to reflect the current value stored in the database.
     */
    public void ejbLoad() {
        System.out.println("ejbLoad() called.");

        Connection dbConn = null;
        PreparedStatement selectStatement = null;
        try {
            Long id = (Long) mEntityContext.getPrimaryKey();
            dbConn = getConnection();
            selectStatement = dbConn.prepareStatement(
                "select * from T_USER where ID = ?");
            selectStatement.setString(1, id.toString());
            ResultSet rs = selectStatement.executeQuery();
            rs.next();
            mId = new Long(rs.getLong("ID"));
            mLogin = rs.getString("LOGIN");
            mPassword = rs.getString("PASSWORD");
            mFullName = rs.getString("FULL_NAME");
        } catch (SQLException sqle) {
            throw new EJBException(sqle.getMessage(), sqle);
        } finally {
            try {
                if (selectStatement != null)
                    selectStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Called from the Container. Updates the database to reflect the current
     * values of this in-memory entity bean instance.
     */
    public void ejbStore() {
        System.out.println("ejbStore() called.");

        Connection dbConn = null;
        PreparedStatement updateStatement = null;
        try {
            Long id = (Long) mEntityContext.getPrimaryKey();
            dbConn = getConnection();
            updateStatement = dbConn.prepareStatement(
                "update T_USER set LOGIN = ?, PASSWORD = ?, FULL_NAME = ? where ID = ?");
            updateStatement.setString(1, mLogin);
            updateStatement.setString(2, mPassword);
            updateStatement.setString(3, mFullName);
            updateStatement.setString(4, id.toString());
            updateStatement.executeUpdate();
        } catch (SQLException sqle) {
            throw new EJBException(sqle.getMessage(), sqle);
        } finally {
            try {
                if (updateStatement != null)
                    updateStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Called by Container. Implementation can acquire needed resources.
     */
    public void ejbActivate() throws EJBException {
        System.out.println("ejbActivate() called.");
        mId = (Long) mEntityContext.getPrimaryKey();
    }

    /**
     * Called by Container. Releases held resources for passivation.
     */
    public void ejbPassivate() throws EJBException {
        System.out.println("ejbPassivate () called.");
        mId = null;
    }

    /**
     * Finds a User by its primary key.
     */
    public Long ejbFindByPrimaryKey(Long aKey)
            throws FinderException {
        System.out.println("ejbFindByPrimaryKey(" + aKey + ") called.");

        Connection dbConn = null;
        PreparedStatement selectStatement = null;
        try {
            dbConn = getConnection();
            selectStatement = dbConn.prepareStatement(
                "select ID from T_USER where ID = ?");
            selectStatement.setString(1, aKey.toString());
            ResultSet rs = selectStatement.executeQuery();
            if (!rs.next())
                throw new ObjectNotFoundException("Could not find User with ID " + aKey);
            return aKey;
        } catch (SQLException sqle) {
            throw new EJBException(sqle.getMessage(), sqle);
        } finally {
            try {
                if (selectStatement != null)
                    selectStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Finds a User by login.
     */
    public Long ejbFindByLogin(String aLogin)
            throws FinderException {
        System.out.println("ejbFindByLogin(" + aLogin + ") called.");

        Connection dbConn = null;
        PreparedStatement selectStatement = null;
        try {
            dbConn = getConnection();
            selectStatement = dbConn.prepareStatement(
                "select ID from T_USER where LOGIN = ?");
            selectStatement.setString(1, aLogin);
            ResultSet rs = selectStatement.executeQuery();
            if (!rs.next())
                throw new ObjectNotFoundException("Can not find User with login=" + aLogin);
            Long id = new Long(rs.getLong(1));
            return id;
        } catch (SQLException sqle) {
            throw new EJBException(sqle.getMessage(), sqle);
        } finally {
            try {
                if (selectStatement != null)
                    selectStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Finds all users.
     */
    public Collection ejbFindAll()
            throws FinderException {
        System.out.println("ejbFindAll() called.");

        Connection dbConn = null;
        Statement selectStatement = null;
        try {
            dbConn = getConnection();
            selectStatement = dbConn.createStatement();
            ResultSet rs = selectStatement.executeQuery("select ID from T_USER");
            ArrayList resultUserIdList = new ArrayList();
            while (rs.next()) {
                Long id = new Long(rs.getString("ID"));
                resultUserIdList.add(id);
            }
            return resultUserIdList;
        } catch (SQLException sqle) {
            throw new FinderException(sqle.getMessage());
        } finally {
            try {
                if (selectStatement != null)
                    selectStatement.close();
            } catch (Exception e) { }
            try {
                if (dbConn != null)
                    dbConn.close();
            } catch (Exception e) { }
        }
    }

    /**
     * Gets connection from the Container's datasource.
     */
    private Connection getConnection()
            throws SQLException {
        DataSource ds = null;
        try {
            Context context = new InitialContext();
            ds = (DataSource) context.lookup(DATA_SOURCE);
            Connection dbConnection = ds.getConnection();
            return dbConnection;
        } catch (NamingException ne) {
            throw new SQLException("Can not find datasource " + DATA_SOURCE);
        }
    }

}
